import { MailSetEntryTypeRef, MailTypeRef } from "../../../common/api/entities/tutanota/TypeRefs.js";
import { DropDownSelector } from "../../../common/gui/base/DropDownSelector.js";
import m from "mithril";
import { TextField } from "../../../common/gui/base/TextField.js";
import { Dialog } from "../../../common/gui/base/Dialog.js";
import { locator } from "../../../common/api/main/CommonLocator.js";
import { LockedError } from "../../../common/api/common/error/RestError.js";
import { lang } from "../../../common/misc/LanguageViewModel.js";
import { MailSetKind } from "../../../common/api/common/TutanotaConstants.js";
import { elementIdPart, isSameId, listIdPart } from "../../../common/api/common/utils/EntityUtils.js";
import { reportMailsAutomatically } from "./MailReportDialog.js";
import { isOfflineError } from "../../../common/api/common/utils/ErrorUtils.js";
import { groupByAndMap } from "@tutao/tutanota-utils";
import { mailLocator } from "../../mailLocator.js";
import { assertNotNull } from "@tutao/tutanota-utils";
import { getFolderName, getIndentedFolderNameForDropdown, getPathToFolderString } from "../model/MailUtils.js";
import { isSpamOrTrashFolder } from "../model/MailChecks.js";
/**
 * Dialog for Edit and Add folder are the same.
 * @param editedFolder if this is null, a folder is being added, otherwise a folder is being edited
 */
export async function showEditFolderDialog(mailBoxDetail, editedFolder = null, parentFolder = null) {
    const noParentFolderOption = lang.get("comboBoxSelectionNone_msg");
    const mailGroupId = mailBoxDetail.mailGroup._id;
    const folders = await mailLocator.mailModel.getMailboxFoldersForId(assertNotNull(mailBoxDetail.mailbox.folders)._id);
    let folderNameValue = editedFolder?.name ?? "";
    let targetFolders = folders
        .getIndentedList(editedFolder)
        // filter: SPAM and TRASH and descendants are only shown if editing (folders can only be moved there, not created there)
        .filter((folderInfo) => !(editedFolder === null && isSpamOrTrashFolder(folders, folderInfo.folder)))
        .map((folderInfo) => {
        return {
            name: getIndentedFolderNameForDropdown(folderInfo),
            value: folderInfo.folder,
        };
    });
    targetFolders = [{ name: noParentFolderOption, value: null }, ...targetFolders];
    let selectedParentFolder = parentFolder;
    let form = () => [
        m(TextField, {
            label: editedFolder ? "rename_action" : "folderName_label",
            value: folderNameValue,
            oninput: (newInput) => {
                folderNameValue = newInput;
            },
        }),
        m(DropDownSelector, {
            label: "parentFolder_label",
            items: targetFolders,
            selectedValue: selectedParentFolder,
            selectedValueDisplay: selectedParentFolder ? getFolderName(selectedParentFolder) : noParentFolderOption,
            selectionChangedHandler: (newFolder) => (selectedParentFolder = newFolder),
            helpLabel: () => (selectedParentFolder ? getPathToFolderString(folders, selectedParentFolder) : ""),
        }),
    ];
    async function getMailIdsGroupedByListId(folder) {
        const mailSetEntries = await locator.entityClient.loadAll(MailSetEntryTypeRef, folder.entries);
        return groupByAndMap(mailSetEntries, (mse) => listIdPart(mse.mail), (mse) => elementIdPart(mse.mail));
    }
    async function loadAllMailsOfFolder(folder, reportableMails) {
        if (folder.isMailSet) {
            const mailIdsPerBag = await getMailIdsGroupedByListId(folder);
            for (const [mailListId, mailIds] of mailIdsPerBag) {
                reportableMails.push(...(await locator.entityClient.loadMultiple(MailTypeRef, mailListId, mailIds)));
            }
        }
        else {
            reportableMails.push(...(await locator.entityClient.loadAll(MailTypeRef, folder.mails)));
        }
    }
    const okAction = async (dialog) => {
        // closing right away to prevent duplicate actions
        dialog.close();
        try {
            // if folder is null, create new folder
            if (editedFolder === null) {
                await locator.mailFacade.createMailFolder(folderNameValue, selectedParentFolder?._id ?? null, mailGroupId);
            }
            else {
                // if it is being moved to trash (and not already in trash), ask about trashing
                if (selectedParentFolder?.folderType === MailSetKind.TRASH && !isSameId(selectedParentFolder._id, editedFolder.parentFolder)) {
                    const confirmed = await Dialog.confirm(() => lang.get("confirmDeleteCustomFolder_msg", {
                        "{1}": getFolderName(editedFolder),
                    }));
                    if (!confirmed)
                        return;
                    await locator.mailFacade.updateMailFolderName(editedFolder, folderNameValue);
                    await mailLocator.mailModel.trashFolderAndSubfolders(editedFolder);
                }
                else if (selectedParentFolder?.folderType === MailSetKind.SPAM && !isSameId(selectedParentFolder._id, editedFolder.parentFolder)) {
                    // if it is being moved to spam (and not already in spam), ask about reporting containing emails
                    const confirmed = await Dialog.confirm(() => lang.get("confirmSpamCustomFolder_msg", {
                        "{1}": getFolderName(editedFolder),
                    }));
                    if (!confirmed)
                        return;
                    // get mails to report before moving to mail model
                    const descendants = folders.getDescendantFoldersOfParent(editedFolder._id).sort((l, r) => r.level - l.level);
                    let reportableMails = [];
                    await loadAllMailsOfFolder(editedFolder, reportableMails);
                    for (const descendant of descendants) {
                        await loadAllMailsOfFolder(descendant.folder, reportableMails);
                    }
                    await reportMailsAutomatically("1" /* MailReportType.SPAM */, locator.mailboxModel, mailLocator.mailModel, mailBoxDetail, reportableMails);
                    await locator.mailFacade.updateMailFolderName(editedFolder, folderNameValue);
                    await mailLocator.mailModel.sendFolderToSpam(editedFolder);
                }
                else {
                    await locator.mailFacade.updateMailFolderName(editedFolder, folderNameValue);
                    await locator.mailFacade.updateMailFolderParent(editedFolder, selectedParentFolder?._id || null);
                }
            }
        }
        catch (error) {
            if (isOfflineError(error) || !(error instanceof LockedError)) {
                throw error;
            }
        }
    };
    Dialog.showActionDialog({
        title: editedFolder ? lang.get("editFolder_action") : lang.get("addFolder_action"),
        child: form,
        validator: () => checkFolderName(mailBoxDetail, folders, folderNameValue, mailGroupId, selectedParentFolder?._id ?? null),
        allowOkWithReturn: true,
        okAction: okAction,
    });
}
function checkFolderName(mailboxDetail, folders, name, mailGroupId, parentFolderId) {
    if (name.trim() === "") {
        return "folderNameNeutral_msg";
    }
    else if (folders.getCustomFoldersOfParent(parentFolderId).some((f) => f.name === name)) {
        return "folderNameInvalidExisting_msg";
    }
    else {
        return null;
    }
}
//# sourceMappingURL=EditFolderDialog.js.map